local projection = require ( 'projection3' )
local vec3 = require ( 'vec3' )

local camera
local object

function love.load ( )
	camera = { }
	camera.projection = projection ( )
	camera.position = vec3 ( 2, 2, 2 )
	camera.target = vec3 ( 0, 0, 0 )
	camera.up = vec3 ( 0, 1, 0 )
	camera.halfwidth = love.graphics.getWidth ( ) / 2
	camera.halfheight = love.graphics.getHeight ( ) / 2

	object = { }
	object.position1 = vec3 ( 0, 0, 0 )
	object.position2 = vec3 ( 0, 0, 0 )
	object.position = vec3 ( 0, 0, 0 )
	object.travel = 1
	object.sprite = love.graphics.newImage ( "sprite.png" )
	object.size = ( camera.halfheight / object.sprite:getWidth ( ) ) * 0.3
	object.center = { }
	object.center.x = object.sprite:getWidth ( ) / 2
	object.center.y = object.sprite:getHeight ( ) / 2
end

function love.update ( dt )
	object.travel = object.travel + dt / 2
	if object.travel >= 1 then
		object.position1:copy ( object.position2 )
		object.position2.x = math.random ( )
		object.position2.y = math.random ( )
		object.position2.z = math.random ( )
		object.travel = object.travel - 1
	end
	object.position:lerp ( object.position1, object.position2, object.travel )

	local x, y = love.mouse.getPosition ( )
	x = ( x - camera.halfwidth ) / camera.halfwidth * math.pi * 2
	y = ( y - camera.halfheight ) / camera.halfheight
	camera.position.x = math.sin ( x ) * 2
	camera.position.z = math.cos ( x ) * 2
	camera.position.y = y * 2
end

function love.draw ( )
	camera.projection:perspective ( math.pi / 2 )
	camera.projection:lookAt ( camera.position, camera.target, camera.up )

	-- red green and blue lines
	local origin = { [0] = vec3 ( 0, 0, 0 ), vec3 ( 1, 0, 0 ), vec3 ( 0, 1, 0 ), vec3 ( 0, 0, 1 ) }
	camera.projection:toScreen ( origin[ 0 ], origin[ 0 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 1 ], origin[ 1 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 2 ], origin[ 2 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 3 ], origin[ 3 ], camera.halfwidth, camera.halfheight )

	love.graphics.setColor ( 255, 0, 0 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 1 ].x, origin[ 1 ].y )
	love.graphics.setColor ( 0, 255, 0 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 2 ].x, origin[ 2 ].y )
	love.graphics.setColor ( 0, 0, 255 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 3 ].x, origin[ 3 ].y )

	origin[ 0 ]:copy ( object.position )
	origin[ 1 ]:copy ( object.position )
	origin[ 2 ]:copy ( object.position )
	origin[ 3 ]:copy ( object.position )
	origin[ 1 ].x = 0
	origin[ 2 ].y = 0
	origin[ 3 ].z = 0
	camera.projection:toScreen ( origin[ 0 ], origin[ 0 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 1 ], origin[ 1 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 2 ], origin[ 2 ], camera.halfwidth, camera.halfheight )
	camera.projection:toScreen ( origin[ 3 ], origin[ 3 ], camera.halfwidth, camera.halfheight )

	love.graphics.setColor ( 255, 0, 0, 100 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 1 ].x, origin[ 1 ].y )
	love.graphics.setColor ( 0, 255, 0, 100 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 2 ].x, origin[ 2 ].y )
	love.graphics.setColor ( 0, 0, 255, 100 )
	love.graphics.line ( origin[ 0 ].x, origin[ 0 ].y, origin[ 3 ].x, origin[ 3 ].y )

	-- actual sprite
	local objectposition = vec3 ( )
	camera.projection:toScreen ( objectposition, object.position, camera.halfwidth, camera.halfheight )
	local objectsize = object.size / objectposition.z

	love.graphics.setColor ( 255, 255, 255, 255 )
	love.graphics.draw ( object.sprite, objectposition.x, objectposition.y, 0, objectsize, objectsize, object.center.x, object.center.y )
end
