--[[
Copyright (c) 2017 raidho36/rcoaxil

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
]]--

-- JIT version requires LuaJIT running
assert ( type ( jit ) == 'table' and jit.status ( ) == true )

Vec3 = require ( 'vec3' )

local mtan, mpi = math.tan, math.pi

local Projection3 = setmetatable ( { }, { __call = function ( class, ... ) return class.new ( ... ) end } )
Projection3.__index = Projection3

function Projection3.new ( )
	local self = setmetatable ( { }, Projection3 )
	self.x = Vec3 ( )
	self.y = Vec3 ( )
	self.z = Vec3 ( )
	self.o = Vec3 ( )
	self.fov = 0
	return self
end

function Projection3:identity ( )
	self.a.x, self.b.x, self.c.x, self.o.x = 1, 0, 0, 0
	self.a.y, self.b.y, self.c.y, self.o.y = 0, 1, 0, 0
	self.a.z, self.b.z, self.c.z, self.o.z = 0, 0, 1, 0
	return self
end

function Projection3:perspective ( fov )
	self.fov = mtan ( ( mpi + fov ) / 2 )
	return self
end

function Projection3:orthogonal ( )
	self.fov = 0
	return self
end

function Projection3:lookAt ( origin, target, up )
	self.o:copy  ( origin )
	self.z:diff  ( target, origin ):normalize ( )
	self.x:cross ( up,     self.z ):normalize ( )
	self.y:cross ( self.z, self.x )
	return self
end

function Projection3:toScreen ( t, v, hw, hh )
	t.x, t.y, t.z = v.x - self.o.x, v.y - self.o.y, v.z - self.o.z

	local z = self.z.x * t.x + self.z.y * t.y + self.z.z * t.z
	local s = self.fov ~= 0 and self.fov / z or 1
	t.x, t.y, t.z =
		( self.x.x * t.x + self.x.y * t.y + self.x.z * t.z ) * s * hh + hw,
		( self.y.x * t.x + self.y.y * t.y + self.y.z * t.z ) * s * hh + hh,
		z

	return t
end

function Projection3:toWorld ( v, hw, hh )
	local s = self.fov ~= 0 and v.z / self.fov or 1
	v.x = ( v.x - hw ) / ( hh * s )
	v.y = ( v.y - hh ) / ( hh * s )

end

local ffi = require ( 'ffi' )

ffi.cdef ( "typedef struct rsml_proj3 { rsml_vec3 x, y, z, o; float fov; } rsml_proj3;" )
Projection3.new = ffi.typeof ( "rsml_proj3" )
ffi.metatype ( "rsml_proj3", Projection3 )

return Projection3
